/*
 * Copyright 2002-2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.didion.vfs.provider.crypt;

import org.apache.commons.vfs.provider.compressed.CompressedFileFileSystem;
import org.apache.commons.vfs.FileName;
import org.apache.commons.vfs.FileObject;
import org.apache.commons.vfs.FileSystemOptions;
import org.apache.commons.vfs.FileSystemException;

import javax.crypto.CipherInputStream;
import javax.crypto.CipherOutputStream;
import javax.crypto.Cipher;
import java.util.Collection;
import java.io.InputStream;
import java.io.OutputStream;

public class CryptFileSystem extends CompressedFileFileSystem {
    private CipherInfo cipherInfo;
    private Cipher encryptCipher;
    private Cipher decryptCipher;

    public CryptFileSystem(
            FileName name,
            FileObject file,
            FileSystemOptions options,
            CipherInfo info)
            throws FileSystemException {
        super(name, file, options);
        this.cipherInfo = info;
    }

    protected void addCapabilities(final Collection caps) {
        caps.addAll(CryptFileProvider.capabilities);
    }

    protected FileObject createFile(final FileName name)
            throws FileSystemException {
        return new CryptFileObject(name, getParentLayer(), this);
    }

    InputStream getDecryptInputStream(InputStream wrapped) throws Exception {
        return new CipherInputStream(wrapped, getDecryptCipher());
    }

    OutputStream getEncryptOutputStream(OutputStream wrapped) throws Exception {
        return new CipherOutputStream(wrapped, getEncryptCipher());
    }

    private Cipher getDecryptCipher() throws Exception {
        if (null == this.decryptCipher) {
            this.decryptCipher = this.cipherInfo.createCipher(Cipher.DECRYPT_MODE);
        }
        return this.decryptCipher;
    }

    private Cipher getEncryptCipher() throws Exception {
        if (null == this.encryptCipher) {
            this.encryptCipher = this.cipherInfo.createCipher(Cipher.ENCRYPT_MODE);
        }
        return this.encryptCipher;
    }
}